@@ -19,7 +19,7 @@
 #' \donttest{
 #' # Initiate GRASS session
 #' if(.Platform$OS.type == "windows"){
-#'   gisbase = "c:/Program Files/GRASS GIS 7.4.0"
+#'   gisbase = "c:/Program Files/GRASS GIS 7.6"
 #'   } else {
 #'   gisbase = "/usr/lib/grass74/"
 #'   }
@@ -37,11 +37,12 @@
 #' # Derive streams from DEM
 #' derive_streams(burn = 0, accum_threshold = 700, condition = TRUE, clean = TRUE)
 #'
-#' # Check and correct complex junctions (there are no complex juctions in this 
-#' # example date set)
-#' cj <- check_compl_junctions()
+#' # Check and correct complex confluences (there are no complex confluences in this
+#' # example date set; set accum_threshold in derive_streams to a smaller value
+#' # to create complex confluences)
+#' cj <- check_compl_confluences()
 #' if(cj){
-#'   correct_compl_junctions()
+#'   correct_compl_confluences()
 #' }
 #' 
 #' # Prepare edges
@@ -72,7 +73,9 @@ calc_binary <- function(){
                parameters = list(
                sql = 'select rid,stream,next_str,prev_str01,prev_str02,netID from edges',
                separator = ','
-               ), intern = TRUE)
+               ), 
+               ignore.stderr = TRUE,
+               intern = TRUE)
 
   dt.streams<-do.call(rbind,strsplit(dt.streams,split=","))
   dt.streams<-apply(dt.streams,2,as.numeric)
@@ -91,34 +94,15 @@ calc_binary <- function(){
   return(bins)
 }
 
-
-# #' Recursive function to assign binary id to stream segments
-# #' data.table dt.streams must be present containing topological information for the edges
-# #' Should be run for all outlets in the network ( = most downstream segments) and fills the binID for all segments
-# #' '<<-' "call by reference" assigns to global variable
-# #' Must be defined within function, otherwise it does not know dt.streams
-# #' @param id: stream segment
-# #' @param binID: binary ID
-# #' @param lastbit: last char to be added (0 or 1)
-# #' @keywords internal
-# assign_binIDs<-function(id,binID,lastbit){
-#   if(dt.streams[id]$prev_str01 == 0){  # check only one of prev01 and prev02 because they are always both 0
-#     dt.streams[id]$binaryID <<- paste0(binID,lastbit)
-#   } else {
-#     dt.streams[id]$binaryID <<- paste0(binID,lastbit)
-#     assign_binIDs(dt.streams[id]$prev_str01,dt.streams[id]$binaryID,0)
-#     assign_binIDs(dt.streams[id]$prev_str02,dt.streams[id]$binaryID,1)
-#   }
-# }
-#
+#' assign_binIDs
 #' Recursive function to assign binary id to stream segments
+#' 
 #' Should be run for all outlets in the network ( = most downstream segments) and fills the binID for all segments
 #' @param id: stream segment
 #' @param binID: binary ID
 #' @param lastbit: last char to be added (0 or 1)
 #' @keywords internal
-#'
-# MiKatt
+#' 
 assign_binIDs <- function(dt, id, binID, lastbit){
   if(dt[stream == id, prev_str01 ] == 0){  # check only one of prev01 and prev02 because they are always both 0
     dt[stream == id, binaryID := paste0(binID, lastbit)]
@@ -129,38 +113,3 @@ assign_binIDs <- function(dt, id, binID, lastbit){
   }
 }
 
-
-# #' workhorse for calc_binary
-# #' @param network network ID
-# #' @keywords internal
-#
-# calc_binary_horse <- function(network) {
-#    # empty id cols
-#    network$bin_id <- rep(NA, nrow(network))
-#    # for each topological dimension
-#    for (i in sort(unique(network$topo_dim))) {
-#      rows <- which(network$topo_dim == i)
-#      # first segment set to one
-#      if (i == 1) {
-#        network$bin_id[rows] <- 1
-#      } else {
-#        # actual segments
-#        take_segments <- network[rows, ]@data
-#        # downstream segments
-#        take_down <- network[network$topo_dim == i - 1, ]@data
-#        names(take_down)[2] <- 'stream_down'
-#        # merge dwn (with bin_id) and actual
-#        take_merge <- merge(take_down[ , c('stream_down', 'bin_id')],
-#                            take_segments, by.x = 'stream_down', by.y = 'next_str')
-#        # assign 0/1 and paste with downstream id
-#        take_merge[ , 'bin_id'] <-  c(aggregate(bin_id.x ~ stream_down, data = take_merge,
-#                  FUN = function(x) paste0(x, sample(c(0, 1), 2)))[ , 'bin_id.x'])
-#        take_merge$bin_id.x <- NULL
-#        take_merge$bin_id.y <- NULL
-#        network$bin_id[rows] <- take_merge$bin_id
-#      }
-#    }
-#    out <- network@data[ , c('rid', 'bin_id')]
-#    names(out) <- c("rid", "binaryID")
-#    return(out)
-#  }
